# Form implementation generated from reading ui file './client/gui/client_gui.ui'
#
# Created by: PyQt6 UI code generator 6.4.0
#
# WARNING: Any manual changes made to this file will be lost when pyuic6 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt6 import QtCore, QtWidgets


class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        MainWindow.setObjectName("MainWindow")
        MainWindow.resize(1000, 800)
        sizePolicy = QtWidgets.QSizePolicy(
            QtWidgets.QSizePolicy.Policy.Preferred, QtWidgets.QSizePolicy.Policy.Preferred
        )
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(MainWindow.sizePolicy().hasHeightForWidth())
        MainWindow.setSizePolicy(sizePolicy)
        self.centralwidget = QtWidgets.QWidget(MainWindow)
        self.centralwidget.setObjectName("centralwidget")
        self.horizontalLayout = QtWidgets.QHBoxLayout(self.centralwidget)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.widgetMain = QtWidgets.QWidget(self.centralwidget)
        self.widgetMain.setObjectName("widgetMain")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.widgetMain)
        self.verticalLayout.setObjectName("verticalLayout")
        self.groupBoxConnection = QtWidgets.QGroupBox(self.widgetMain)
        self.groupBoxConnection.setObjectName("groupBoxConnection")
        self.gridLayout = QtWidgets.QGridLayout(self.groupBoxConnection)
        self.gridLayout.setObjectName("gridLayout")
        self.lineEditIP = QtWidgets.QLineEdit(self.groupBoxConnection)
        self.lineEditIP.setObjectName("lineEditIP")
        self.gridLayout.addWidget(self.lineEditIP, 1, 0, 1, 1)
        self.labelIP = QtWidgets.QLabel(self.groupBoxConnection)
        self.labelIP.setObjectName("labelIP")
        self.gridLayout.addWidget(self.labelIP, 0, 0, 1, 1)
        self.pushButtonConnect = QtWidgets.QPushButton(self.groupBoxConnection)
        self.pushButtonConnect.setObjectName("pushButtonConnect")
        self.gridLayout.addWidget(self.pushButtonConnect, 1, 4, 1, 1)
        self.labelPort = QtWidgets.QLabel(self.groupBoxConnection)
        self.labelPort.setObjectName("labelPort")
        self.gridLayout.addWidget(self.labelPort, 0, 1, 1, 1)
        self.lineEditUsername = QtWidgets.QLineEdit(self.groupBoxConnection)
        self.lineEditUsername.setObjectName("lineEditUsername")
        self.gridLayout.addWidget(self.lineEditUsername, 1, 2, 1, 1)
        self.labelUsername = QtWidgets.QLabel(self.groupBoxConnection)
        self.labelUsername.setObjectName("labelUsername")
        self.gridLayout.addWidget(self.labelUsername, 0, 2, 1, 1)
        self.lineEditPort = QtWidgets.QLineEdit(self.groupBoxConnection)
        self.lineEditPort.setObjectName("lineEditPort")
        self.gridLayout.addWidget(self.lineEditPort, 1, 1, 1, 1)
        self.lineEditPasswd = QtWidgets.QLineEdit(self.groupBoxConnection)
        self.lineEditPasswd.setEchoMode(QtWidgets.QLineEdit.EchoMode.Password)
        self.lineEditPasswd.setObjectName("lineEditPasswd")
        self.gridLayout.addWidget(self.lineEditPasswd, 1, 3, 1, 1)
        self.labelPasswd = QtWidgets.QLabel(self.groupBoxConnection)
        self.labelPasswd.setObjectName("labelPasswd")
        self.gridLayout.addWidget(self.labelPasswd, 0, 3, 1, 1)
        self.verticalLayout.addWidget(self.groupBoxConnection)
        self.groupBoxChat = QtWidgets.QGroupBox(self.widgetMain)
        self.groupBoxChat.setObjectName("groupBoxChat")
        self.verticalLayout_4 = QtWidgets.QVBoxLayout(self.groupBoxChat)
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.textBrowserChat = QtWidgets.QTextBrowser(self.groupBoxChat)
        self.textBrowserChat.setObjectName("textBrowserChat")
        self.verticalLayout_4.addWidget(self.textBrowserChat)
        self.textEditMessage = QtWidgets.QTextEdit(self.groupBoxChat)
        self.textEditMessage.setMaximumSize(QtCore.QSize(16777215, 150))
        self.textEditMessage.setObjectName("textEditMessage")
        self.verticalLayout_4.addWidget(self.textEditMessage)
        self.pushButtonSend = QtWidgets.QPushButton(self.groupBoxChat)
        self.pushButtonSend.setObjectName("pushButtonSend")
        self.verticalLayout_4.addWidget(self.pushButtonSend)
        self.verticalLayout.addWidget(self.groupBoxChat)
        self.horizontalLayout.addWidget(self.widgetMain)
        self.widgetContacts = QtWidgets.QWidget(self.centralwidget)
        self.widgetContacts.setMaximumSize(QtCore.QSize(300, 16777215))
        self.widgetContacts.setObjectName("widgetContacts")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.widgetContacts)
        self.verticalLayout_3.setSizeConstraint(QtWidgets.QLayout.SizeConstraint.SetDefaultConstraint)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.groupBoxContacts = QtWidgets.QGroupBox(self.widgetContacts)
        self.groupBoxContacts.setObjectName("groupBoxContacts")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.groupBoxContacts)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.listWidgetContacts = QtWidgets.QListWidget(self.groupBoxContacts)
        self.listWidgetContacts.setObjectName("listWidgetContacts")
        self.verticalLayout_2.addWidget(self.listWidgetContacts)
        self.pushButtonAddContact = QtWidgets.QPushButton(self.groupBoxContacts)
        self.pushButtonAddContact.setObjectName("pushButtonAddContact")
        self.verticalLayout_2.addWidget(self.pushButtonAddContact)
        self.pushButtonDeleteContact = QtWidgets.QPushButton(self.groupBoxContacts)
        self.pushButtonDeleteContact.setObjectName("pushButtonDeleteContact")
        self.verticalLayout_2.addWidget(self.pushButtonDeleteContact)
        self.verticalLayout_3.addWidget(self.groupBoxContacts)
        self.horizontalLayout.addWidget(self.widgetContacts)
        MainWindow.setCentralWidget(self.centralwidget)

        self.retranslateUi(MainWindow)
        QtCore.QMetaObject.connectSlotsByName(MainWindow)
        MainWindow.setTabOrder(self.lineEditUsername, self.lineEditPasswd)
        MainWindow.setTabOrder(self.lineEditPasswd, self.pushButtonConnect)
        MainWindow.setTabOrder(self.pushButtonConnect, self.listWidgetContacts)
        MainWindow.setTabOrder(self.listWidgetContacts, self.textEditMessage)
        MainWindow.setTabOrder(self.textEditMessage, self.pushButtonSend)
        MainWindow.setTabOrder(self.pushButtonSend, self.pushButtonAddContact)
        MainWindow.setTabOrder(self.pushButtonAddContact, self.pushButtonDeleteContact)
        MainWindow.setTabOrder(self.pushButtonDeleteContact, self.textBrowserChat)
        MainWindow.setTabOrder(self.textBrowserChat, self.lineEditIP)
        MainWindow.setTabOrder(self.lineEditIP, self.lineEditPort)

    def retranslateUi(self, MainWindow):
        _translate = QtCore.QCoreApplication.translate
        MainWindow.setWindowTitle(_translate("MainWindow", "GB PyChat"))
        self.groupBoxConnection.setTitle(_translate("MainWindow", "Настройки подключения"))
        self.lineEditIP.setPlaceholderText(_translate("MainWindow", "127.0.0.1"))
        self.labelIP.setText(_translate("MainWindow", "IP адрес сервера:"))
        self.pushButtonConnect.setText(_translate("MainWindow", "Подключиться"))
        self.labelPort.setText(_translate("MainWindow", "Порт сервера:"))
        self.labelUsername.setText(_translate("MainWindow", "Имя пользователя:"))
        self.lineEditPort.setPlaceholderText(_translate("MainWindow", "7777"))
        self.labelPasswd.setText(_translate("MainWindow", "Пароль:"))
        self.groupBoxChat.setTitle(_translate("MainWindow", "Чат с пользователем"))
        self.pushButtonSend.setText(_translate("MainWindow", "Отправить"))
        self.groupBoxContacts.setTitle(_translate("MainWindow", "Контакты"))
        self.pushButtonAddContact.setText(_translate("MainWindow", "Добавить контакт"))
        self.pushButtonDeleteContact.setText(_translate("MainWindow", "Удалить контакт"))


if __name__ == "__main__":
    import sys

    app = QtWidgets.QApplication(sys.argv)
    MainWindow = QtWidgets.QMainWindow()
    ui = Ui_MainWindow()
    ui.setupUi(MainWindow)
    MainWindow.show()
    sys.exit(app.exec())
